#! /usr/bin/env python
"""
--------------------------------------------------------------------------------
Copyright (C) 2005 Sylvain Fourmanoit <syfou@users.sourceforge.net>
        for the code
Copyright (C) 2004 Brian Cornell, author of gdesklets' psi-xmms display
	for most of the artwork.

Released under the GPL, version 2. Except the artwork, which is released
pursuant to the restrictions imposed by their authors.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
--------------------------------------------------------------------------------
This is a real life example of a working interactive desklet able to control
the xmms player.

It was built as a proof of concept, a minimal show-case of adesklets
capabilities. I used the itertools module to simplify my work, so you need
at least python 2.3 to run the code (See README for all limitions)

This was tested on python 2.4 final, on a linux 2.6.10 glibc 2.3.4
x86 machine.

Just run this script from its final location on you disk to start the desklet.

--------------------------------------------------------------------------------
"""
import adesklets
from os.path import dirname

#-------------------------------------------------------------------------------
class Events(adesklets.Events_handler):
    """
    Our event class. Interesting as a interactive desklet demo
    """
    import itertools as it
    
    icons = [ ('prev.png',None),
              ('playoff.png','playon.png'),
              ('pauseoff.png','pauseon.png'),
              ('stopoff.png','stopon.png'),
              ('next.png',None),
              ('eject.png',None),
              ('randomoff.png','randomon.png'),
              ('repeatoff.png','repeaton.png'),
              ('hideoff.png','hideon.png')
              ]

    icons_show = True
    icons_display = {}
    window_dim = []
    window_foreground = 0
        
    class _XmmsInstance:
        """
        All xmms-related operations take place through this class.
        It acts both:

        - As a container for player status
        - As a callable for all xmms operations
        """
        import xmms.control as xc
        __status = []
        
        def __init__(self,dir):
            self.__status= self.get_status()
            
            self._actions = [  self.previous,
                               self.play,
                               self.pause,
                               self.stop,
                               self.next,
                               self.eject,
                               self.random,
                               self.repeat,
                               self.hide
                               ]
            
        def get_status(self):
            return [  0,
                      self.xc.is_playing() \
                      and not self.xc.is_paused(),
                      self.xc.is_paused(),
                      not self.xc.is_playing()\
                      and not self.xc.is_paused(),
                      0,
                      0,
                      self.xc.is_shuffle(),
                      self.xc.is_repeat(),
                      not self.xc.is_main_win()
                      ]
        
        def __len__(self):
            return len(self.__status)
        
        def __getitem__(self,key):
            return self.__status[key]
        
        def __setitem__(self,key,value):
            self.__status[key]=value
            
        def __iter__(self):
            return iter(self.__status)
        
        def __str__(self):
            return str(self.__status)

        def __call__(self,key):
            self._actions[key]()
        
        def previous(self):
            self.xc.playlist_prev()
        
        def play(self):
            self.xc.play()
        
        def pause(self):
            if not self.xc.is_paused():
                self.xc.pause()
        
        def stop(self):
            self.xc.stop()
        
        def next(self):
            self.xc.playlist_next()
        
        def eject(self):
            self.xc.eject()
        
        def random(self):
            self.xc.toggle_shuffle()
        
        def repeat(self):
            self.xc.toggle_repeat()

        def hide(self):
            self.xc.main_win_toggle(not self.xc.is_main_win())
    
    def __init__(self, basedir):
        if len(basedir)==0:
            self.basedir='.'
        else:
            self.basedir=basedir
        adesklets.Events_handler.__init__(self)
        
    def __del__(self):
        adesklets.Events_handler.__del__(self)
        
    def ready(self):
        # Initial command status
        #
        self.xmms = Events._XmmsInstance(self.basedir)
        
        # Icons load and caracterisation
        #
        dim = zip(list(self._flatten_filter(self.icons)),
        [ (adesklets.image_get_width(),adesklets.image_get_height())
          for id, item in self.it.izip(self.it.count(2),
                                       self._flatten_filter(self.icons))
          if adesklets.load_image(self.basedir+'/icons/'+item)>=2 and \
          adesklets.context_set_image(id)==None])

        # Icons position computation
        #
	pos = [ item[1] for item in \
              self.it.izip(self._flatten(self.icons),\
                      self._dup(self._pos([item[0] for item in \
                                           [dict(dim)[icon[0]] for icon in \
                                            self.icons]])))\
                if item[0]]

        # ID computation
        #
        id = [item[0] for item in \
             self.it.izip(self.it.count(2),
                          self._flatten_filter(self.icons))]

        # Finally, make a dictionary of all this information
        #
        self.icons_display = dict(zip([item[0] for item in dim], \
                                      zip(id,pos,[item[1] for item in dim])))


        # Set up the initial display
        #
        adesklets.context_set_image(0)
        adesklets.context_set_color(0,0,0,0)
        adesklets.context_set_blend(False)
        self.window_dim=[pos[-1]+dim[-1][1][0]+5, \
                         max([item[1][1] for item in dim])]
        adesklets.window_resize(*self.window_dim)
        self.window_foreground = adesklets.clone_image()
        adesklets.window_set_transparency(True)
        
        for icon,status in zip(self.icons,self.xmms):
            self.draw_icon(icon[status])
            
        adesklets.window_show()

    def enter_notify(self,delayed,x,y):
        self.icons_show=True
        adesklets.context_set_image(0)
        adesklets.blend_image_onto_image(self.window_foreground,1,
                                         0,0,
                                         self.window_dim[0],self.window_dim[1],
                                         0,0,
                                         self.window_dim[0],self.window_dim[1])

    def leave_notify(self,delayed,x,y):
        self.icons_show=False
        adesklets.context_set_image(0)
        adesklets.image_fill_rectangle(0,0,*self.window_dim)
                                                                                 
    def button_press(self, delayed, x, y, button):
        if not delayed and button==1:
            # Get click position
            #
            for icon, (id,pos,dim) in self.icons_display.iteritems():
                if x>=pos-2 and x<=pos+dim[0]+3:
                    break
            for pos,item in self.it.izip(self.it.count(0),self.icons):
                if icon in item:
                    break

            # Compute new status resulting from click
            #
            new_status=list(self.xmms);
            if not (pos in range(1,4)):
                new_status[pos]^=1&(self.icons[pos][1]!=None)
            else:
                new_status[1:4]=[0]*3
                new_status[pos]=1

            # Update icons
            #
            self.update_icons(new_status)
            
            # Take appropriate action
            #
            self.xmms(pos)
        
    def alarm(self):
        self.block()
        self.update_icons(self.xmms.get_status())
        self.unblock()                      
        return 1
    
    def draw_icon(self,icon):
        id,pos,dim = self.icons_display[icon]
        adesklets.context_set_image(self.window_foreground)
        adesklets.image_fill_rectangle(pos,0,0,self.window_dim[1])
        adesklets.blend_image_onto_image(id,1,\
                                         0,0,dim[0],dim[1],\
                                         pos,\
                                         (self.window_dim[1]-dim[1])/2,\
                                         dim[0],dim[1])
        if self.icons_show:
            adesklets.context_set_image(0)
            adesklets.blend_image_onto_image(self.window_foreground,1,
                                             pos,
                                             (self.window_dim[1]-dim[1])/2,
                                             dim[0], dim[1],
                                             pos,
                                             (self.window_dim[1]-dim[1])/2,
                                             dim[0], dim[1])

    def update_icons(self,new_status):
        for id,old,new in self.it.izip(self.it.count(0),
                                       self.xmms,new_status):
            if old!=new:
                self.draw_icon(self.icons[id][new])
        self.xmms[:]=new_status
    
    def _flatten(self,items):
        for item in items:
            for subitem in item:
                yield subitem
                
    def _flatten_filter(self,items):
        """
        Generator to get linear file list
        """
        for item in items:
            for subitem in item:
                if subitem:
                    yield subitem
                    
    def _pos(self, sequence):
        curr=2
        for term in sequence:
            yield curr
            curr+=term+5

    def _dup(self, iterable):
        for term in iterable:
            yield term
            yield term

#-------------------------------------------------------------------------------
Events(dirname(__file__)).pause()
